@@ -1,6 +1,3 @@
-#' @include get_RLum.R set_RLum.R length_RLum.R structure_RLum.R names_RLum.R smooth_RLum.R
-NULL
-
 #' Class `"RLum.Analysis"`
 #'
 #' Object class to represent analysis data for protocol analysis, i.e. all curves,
@@ -25,7 +22,7 @@ NULL
 #' @section Objects from the Class:
 #' Objects can be created by calls of the form `set_RLum("RLum.Analysis", ...)`.
 #'
-#' @section Class version: 0.4.17
+#' @section Class version: 0.4.18
 #'
 #' @author
 #' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
@@ -96,10 +93,7 @@ setAs("list", "RLum.Analysis",
 
 setAs("RLum.Analysis", "list",
       function(from){
-        lapply(1:length(from@records), function(x){
-          from@records[[x]]
-
-        })
+        lapply(from@records, function(x) x)
       })
 
 
@@ -116,8 +110,7 @@ setMethod("show",
             ##print
             cat("\n [RLum.Analysis-class]")
 
-            ##show slot originator, for compatibly reasons with old example data, here
-            ##a check
+            ## show originator, for compatibility reasons with old example data
             if(.hasSlot(object, "originator")){cat("\n\t originator:", paste0(object@originator,"()"))}
 
             cat("\n\t protocol:", object@protocol)
@@ -173,15 +166,12 @@ setMethod("show",
                         } else if (linebreak) {
                           last <- "\n\t .. .. : "
                           assign(x = "linebreak", value = FALSE, envir = env)
-
                         }
-
                       }
                       return(paste0(first,last))
 
                     }else{
                       return("")
-
                     }
 
               }, FUN.VALUE = vector(mode = "character", length = 1))
@@ -195,15 +185,13 @@ setMethod("show",
 
                  } else{
                    cat("\n\t .. .. : ", terminal_output, sep = "")
-
                  }
-
               })
 
             }else{
               cat("\n\t >> This is an empty object, which cannot be used for further analysis! <<")
-
             }
+            cat("\n")
           }
 )##end show method
 
@@ -257,12 +245,12 @@ setMethod(
     ##allow self set to reset an RLum.Analysis object
     if(inherits(records, "RLum.Analysis")){
       #fill slots (this is much faster than the old code!)
-      newRLumAnalysis@protocol <- if(missing(protocol)){records@protocol}else{protocol}
+      newRLumAnalysis@protocol <- if(missing(protocol)) records@protocol else protocol
       newRLumAnalysis@originator <- originator
       newRLumAnalysis@records <- records@records
-      newRLumAnalysis@info <- if(missing(info)){records@info}else{c(records@info, info)}
+      newRLumAnalysis@info <- if(missing(info)) records@info else c(records@info, info)
       newRLumAnalysis@.uid <- .uid
-      newRLumAnalysis@.pid <- if(missing(.pid)){records@.uid}else{.pid}
+      newRLumAnalysis@.pid <- if(missing(.pid)) records@.uid else .pid
 
     }else{
       #fill slots (this is much faster than the old code!)
@@ -272,11 +260,9 @@ setMethod(
       newRLumAnalysis@info <- info
       newRLumAnalysis@.uid <- .uid
       newRLumAnalysis@.pid <- .pid
-
     }
 
     return(newRLumAnalysis)
-
   }
 )
 
@@ -293,7 +279,7 @@ setMethod(
 #' The selection of a specific RLum.type object superimposes the default selection.
 #' Currently supported objects are: RLum.Data.Curve and RLum.Data.Spectrum
 #'
-#' @param object [`get_RLum`]: [`names_RLum`], [`length_RLum`], [`structure_RLum`] (**required**):
+#' @param object (**required**):
 #' an object of class [RLum.Analysis-class]
 #'
 #' @param record.id [`get_RLum`]: [numeric] or [logical] (*optional*):
@@ -314,10 +300,10 @@ setMethod(
 #' return a numeric vector with the index of each element in the RLum.Analysis object.
 #'
 #' @param recursive [`get_RLum`]: [logical] (*with default*):
-#' if `TRUE` (the default) and the result of the `get_RLum()` request is a single
-#' object this object will be unlisted, means only the object itself and no
-#' list containing exactly one object is returned. Mostly this makes things
-#' easier, however, if this method is used within a loop this might be undesired.
+#' if `TRUE` (default) when the result of the `get_RLum()` request is a single
+#' object, the object itself will be returned directly, rather than being
+#' wrapped in a list. Mostly this makes things easier, but this might be
+#' undesired if this method is used within a loop.
 #'
 #' @param drop [`get_RLum`]: [logical] (*with default*):
 #' coerce to the next possible layer (which are `RLum.Data`-objects),
@@ -361,21 +347,20 @@ setMethod("get_RLum",
               info_el <- unique(unlist(lapply(object@records, function(el) names(el@info))))
 
               envir <- as.data.frame(do.call(rbind,
-                                             lapply(object@records, function(el) {
-                                               val <- c(curveType = el@curveType, recordType = el@recordType, unlist(el@info))
+                lapply(object@records, function(el) {
+                  val <- c(curveType = el@curveType, recordType = el@recordType, unlist(el@info))
 
-                                               # add missing info elements and set NA
-                                               if (any(!info_el %in% names(val))) {
-                                                 val_new <- setNames(rep(NA, length(info_el[!info_el %in% names(val)])), info_el[!info_el %in% names(val)])
-                                                 val <- c(val, val_new)
+                  # add missing info elements and set NA
+                  if (any(!info_el %in% names(val))) {
+                    val_new <- setNames(
+                      rep("",length(info_el[!info_el %in% names(val)])), info_el[!info_el %in% names(val)])
+                    val <- c(val, val_new)
+                  }
 
-                                               }
-
-                                               # order the named char vector by its names so we don't mix up the columns
-                                               val <- val[order(names(val))]
-                                               return(val)
-                                             })
-              ), stringsAsFactors = FALSE)
+                 # order the named char vector by its names so we don't mix up the columns
+                 val <- val[order(names(val))]
+                 return(val)
+               })), stringAsFactors = FALSE)
 
               ##select relevant rows
               sel <- tryCatch(eval(
@@ -385,7 +370,7 @@ setMethod("get_RLum",
               ),
               error = function(e) {
                 .throw_error("Invalid subset expression, valid terms are: ",
-                             paste(names(envir), collapse = ", "))
+                             .collapse(names(envir)))
               })
 
               if (!is.logical(sel)) {
@@ -399,13 +384,13 @@ setMethod("get_RLum",
                 object@records <- object@records[sel]
                 return(object)
               } else {
-                tmp <- mapply(function(name, op) { message("  ",name, ": ", paste(unique(op),  collapse = ", ")) },
-                              names(envir), envir)
-                message("\n [get_RLum()] 'subset' expression produced an ",
-                        "empty selection, NULL returned")
+                tmp <- mapply(function(name, op) {
+                  message("  ", name, ": ", .collapse(unique(op), quote = FALSE))
+                }, names(envir), envir)
+                .throw_message("'subset' expression produced an ",
+                               "empty selection, NULL returned")
                 return(NULL)
               }
-
             }
 
             ##if info.object is set, only the info objects are returned
@@ -420,7 +405,7 @@ setMethod("get_RLum",
                                  "objects, NULL returned")
                 }else{
                   .throw_warning("Invalid 'info.object' name, valid names are: ",
-                                 paste(names(object@info), collapse = ", "))
+                                 .collapse(names(object@info)))
                 }
                 return(NULL)
               }
@@ -438,23 +423,20 @@ setMethod("get_RLum",
               if (is.null(record.id)) {
                 record.id <- c(1:length(object@records))
 
-              } else if (!is.numeric(record.id) &
-                         !is.logical(record.id)) {
-                .throw_error("'record.id' has to be of type 'numeric' or ",
-                             "'logical'")
+              } else {
+                .validate_class(record.id, c("integer", "numeric", "logical"))
               }
               ##logical needs a slightly different treatment
               ##Why do we need this? Because a lot of standard R functions work with logical
               ##values instead of numerical indices
               if (is.logical(record.id)) {
                 record.id <- c(1:length(object@records))[record.id]
-
               }
 
               ##check if record.id exists
-              if (FALSE %in% (abs(record.id) %in% (1:length(object@records)))) {
-                message("[get_RLum()] Error: At least one 'record.id' ",
-                        "is invalid, NULL returned")
+              if (!all(abs(record.id) %in% (1:length(object@records)))) {
+                .throw_message("At least one 'record.id' is invalid, ",
+                               "NULL returned")
                 return(NULL)
               }
 
@@ -464,8 +446,8 @@ setMethod("get_RLum",
                   unique(vapply(object@records, function(x)
                     x@recordType, character(1)))
 
-              } else if (!inherits(recordType, "character")){
-                .throw_error("'recordType' has to be of type 'character'")
+              } else {
+                .validate_class(recordType, "character")
               }
 
               ##curveType
@@ -475,32 +457,29 @@ setMethod("get_RLum",
                                                     object@records[[x]]@curveType
                                                   })))
 
-              } else if (!is(curveType, "character")) {
-                .throw_error("'curveType' has to be of type 'character'")
+              } else {
+                .validate_class(curveType, "character")
               }
 
               ##RLum.type
               if (is.null(RLum.type)) {
                 RLum.type <- c("RLum.Data.Curve", "RLum.Data.Spectrum", "RLum.Data.Image")
-
-              } else if (!is(RLum.type, "character")) {
-                .throw_error("'RLum.type' has to be of type 'character'")
+              } else {
+                .validate_class(RLum.type, "character")
               }
 
               ##get.index
               if (is.null(get.index)) {
                 get.index <- FALSE
 
-              } else if (!is(get.index, "logical")) {
-                .throw_error("'get.index' has to be of type 'logical'")
+              } else {
+                .validate_logical_scalar(get.index)
               }
 
               ##get originator
+              originator <- NA_character_
               if (.hasSlot(object, "originator")) {
                 originator <- object@originator
-
-              } else{
-                originator <- NA_character_
               }
 
               ##-----------------------------------------------------------------##
@@ -528,9 +507,7 @@ setMethod("get_RLum",
                       if (grepl(recordType[k], recordType_comp) &
                           object@records[[x]]@curveType %in% curveType) {
                         if (!get.index) object@records[[x]] else x
-
                       }
-
                     })
 
                     ##remove empty entries and select just one to unlist
@@ -573,12 +550,15 @@ setMethod("get_RLum",
 
                     } else{
                       return(temp)
-
                     }
                   }
                 }
 
               } else{
+                ## FIXME(mcol): this block seems unreachable : as before the
+                ## `if` block we set `record.id <- 1:length(object@records)`,
+                ## it can never happen that `length(record.id) < 1`
+                # nocov start
                 if (!get.index[1]) {
                   if (drop == FALSE) {
                     ##needed to keep the argument drop == TRUE
@@ -597,11 +577,10 @@ setMethod("get_RLum",
 
                 } else{
                   return(record.id)
-
                 }
+                # nocov end
               }
             }
-
           })
 
 # structure_RLum() ----------------------------------------------------------------------------
@@ -610,7 +589,7 @@ setMethod("get_RLum",
 #' Method to show the structure of an [RLum.Analysis-class] object.
 #'
 #' @param fullExtent [structure_RLum]; [logical] (*with default*):
-#' extents the returned `data.frame` to its full extent, i.e. all info elements
+#' extends the returned `data.frame` to its full extent, i.e. all info elements
 #' are part of the return as well. The default value is `FALSE` as the data
 #' frame might become rather big.
 #'
@@ -636,7 +615,7 @@ setMethod("structure_RLum",
             temp.object.length <- length(object@records)
 
             ##ID
-            temp.id <- 1:temp.object.length
+            temp.id <- seq_along(object@records)
 
             ##recordType
             temp.recordType <-
@@ -667,8 +646,9 @@ setMethod("structure_RLum",
             temp.uid <- unlist(lapply(object@records, function(x){x@.uid}))
 
             ##.pid
-            temp.pid <- paste(
-              unlist(lapply(object@records, function(x){x@.pid})), collapse = ", ")
+            temp.pid <- unlist(lapply(object@records, function(x){x@.pid}))
+            if (length(temp.pid) > 1)
+              temp.pid <- paste(temp.pid, collapse = ", ")
 
             ##originator
             temp.originator <- unlist(lapply(object@records, function(x){x@originator}))
@@ -690,15 +670,13 @@ setMethod("structure_RLum",
               }
             } else{
               temp.info.elements <-
-                unlist(sapply(1:temp.object.length, function(x) {
-                  if (length(object@records[[x]]@info) != 0) {
-                    paste(names(object@records[[x]]@info), collapse = " ")
+                unlist(sapply(object@records, function(x) {
+                  if (length(x@info) != 0) {
+                    paste(names(x@info), collapse = " ")
                   } else{
                     NA
                   }
-
                 }))
-
             }
 
             ##combine output to a data.frame
@@ -720,7 +698,6 @@ setMethod("structure_RLum",
                 stringsAsFactors = FALSE
               )
             )
-
           })
 
 
@@ -740,12 +717,12 @@ setMethod("length_RLum",
           "RLum.Analysis",
           function(object){
             length(object@records)
-
           })
 
 # names_RLum() --------------------------------------------------------------------------------
 #' @describeIn RLum.Analysis
-#' Returns the names of the [RLum.Data-class] objects objects (same as shown with the show method)
+#' Returns the names of the [RLum.Data-class] objects (same as shown with the
+#' `show` method)
 #'
 #' @return
 #'
@@ -758,9 +735,105 @@ setMethod("length_RLum",
 setMethod("names_RLum",
           "RLum.Analysis",
           function(object){
-            sapply(1:length(object@records), function(x){
-              object@records[[x]]@recordType})
+            sapply(object@records, function(x) x@recordType)
+          })
 
+
+## add_metadata() -----------------------------------------------------------
+#' @describeIn RLum.Analysis
+#' Adds metadata to [RLum.Analysis-class] objects
+#'
+#' @param info_element [character] (**required**) name of the metadata field
+#' to add
+#'
+#' @param value (**required**) The value assigned to the selected element
+#' of the metadata field.
+#'
+#' @keywords internal
+#'
+#' @md
+#' @export
+setMethod("add_metadata<-",
+          signature= "RLum.Analysis",
+          definition = function(object, info_element, value) {
+            .set_function_name("add_metadata")
+            on.exit(.unset_function_name(), add = TRUE)
+
+            ## add the metadata to all records
+            records <- lapply(object@records, function(x) {
+              do.call(`add_metadata<-`,
+                      list(x, info_element = info_element, value = value))
+            })
+
+            object@records <- records
+            assign(x = deparse(substitute(object))[1], object)
+          })
+
+## rename_metadata() --------------------------------------------------------
+#' @describeIn RLum.Analysis
+#' Renames a metadata entry of [RLum.Analysis-class] objects
+#'
+#' @param info_element [character] (**required**) name of the metadata field
+#' to rename
+#'
+#' @param value (**required**) The value assigned to the selected element
+#' of the `info` slot.
+#'
+#' @keywords internal
+#'
+#' @md
+#' @export
+setMethod("rename_metadata<-",
+          signature= "RLum.Analysis",
+          definition = function(object, info_element, value) {
+            .set_function_name("rename_metadata")
+            on.exit(.unset_function_name(), add = TRUE)
+
+            ## rename the metadata field in all records
+            records <- lapply(object@records, function(x) {
+              do.call(`rename_metadata<-`,
+                      list(x, info_element = info_element, value = value))
+            })
+
+            object@records <- records
+            assign(x = deparse(substitute(object))[1], object)
+          })
+
+## replace_metadata() -------------------------------------------------------
+#' @describeIn RLum.Analysis
+#' Replaces or removes metadata of [RLum.Analysis-class] objects
+#'
+#' @param info_element [character] (**required**) name of the metadata field
+#' to replace or remove
+#'
+#' @param value (**required**) The value assigned to the selected elements
+#' of the metadata field. If `NULL` the elements named in `info_element`
+#' will be removed.
+#'
+#' @keywords internal
+#'
+#' @md
+#' @export
+setMethod("replace_metadata<-",
+          signature= "RLum.Analysis",
+          definition = function(object, info_element, subset = NULL, value) {
+            .set_function_name("replace_metadata")
+            on.exit(.unset_function_name(), add = TRUE)
+
+            ## this must be evaluated now, as inside the lapply() inner
+            ## function it would be evaluated differently
+            subset.expr <- substitute(subset)
+
+            ## replace the metadata in all records
+            records <- lapply(object@records, function(x) {
+              do.call(`replace_metadata<-`,
+                      list(x, info_element = info_element,
+                           subset = subset.expr,  verbose = FALSE,
+                           value = value))
+            })
+
+            object@records <- records
+            assign(x = deparse(substitute(object))[1], object)
           })
 
 
@@ -768,8 +841,7 @@ setMethod("names_RLum",
 #' @describeIn RLum.Analysis
 #'
 #' Smoothing of `RLum.Data` objects contained in this `RLum.Analysis` object
-#' [zoo::rollmean] or [zoo::rollmedian][zoo::rollmean]. In particular the internal
-#' function `.smoothing` is used.
+#' using the internal function `.smoothing`.
 #'
 #' @param ... further arguments passed to underlying methods
 #'
@@ -787,9 +859,138 @@ setMethod(
   function(object, ...) {
         object@records <- lapply(object@records, function(x){
           smooth_RLum(x, ...)
-
         })
 
     return(object)
   }
 )
+
+
+## sort_RLum() --------------------------------------------------------------
+#' @describeIn RLum.Analysis
+#'
+#' Sorting of `RLum.Data` objects contained in this `RLum.Analysis` object.
+#' At least one of `slot` and `info_element` must be provided. If both are
+#' given, ordering by `slot` always supersedes ordering by `info_element`.
+#'
+#' @param slot [character] (*optional*): slot name to use in sorting.
+#'
+#' @param info_element [character] (*optional*): name of the `info` field
+#' to use in sorting.
+#'
+#' @param decreasing [logical] (*with default*): whether the sort order should
+#' be decreasing (`FALSE` by default).
+#'
+#' @param ... further arguments passed to underlying methods
+#'
+#' @return
+#'
+#' **`sort_RLum`**
+#'
+#' Same object as input, but sorted according to the specified parameters.
+#'
+#' @md
+#' @export
+setMethod(
+  f = "sort_RLum",
+  signature = "RLum.Analysis",
+  function(object, slot = NULL, info_element = NULL,
+           decreasing = FALSE, ...) {
+    .set_function_name("sort_RLum")
+    on.exit(.unset_function_name(), add = TRUE)
+
+    ## input validation
+    sort.by.slot <- FALSE
+    if (!is.null(slot)) {
+      .validate_class(slot, "character", extra = "NULL")
+      valid.names <- slotNames(object@records[[1]])
+      if (!slot %in% valid.names) {
+        .throw_error("Invalid 'slot' name, valid names are: ",
+                     .collapse(valid.names))
+      }
+      sort.by.slot <- TRUE
+    }
+    if (!is.null(info_element)) {
+      .validate_class(info_element, "character", extra = "NULL")
+      valid.names <- names(object@records[[1]]@info)
+      if (!info_element%in% valid.names) {
+        .throw_error("Invalid 'info_element' name, valid names are: ",
+                     .collapse(valid.names))
+      }
+    }
+    if (is.null(slot) && is.null(info_element)) {
+      .throw_error("At least one of 'slot' and 'info_element' should not be NULL")
+    }
+    .validate_logical_scalar(decreasing)
+
+    ## extract the values from the records according to which the sorting is
+    ## done: ordering by slot always supersedes ordering by info_element
+    vals <- if (sort.by.slot) {
+              sapply(object@records, function(x) slot(x, slot))
+            } else {
+              sapply(object@records, function(x) slot(x, "info")[[info_element]])
+            }
+
+    ## determine the new ordering if possible
+    tryCatch(ord <- order(vals, decreasing = decreasing),
+             error = function(e) {
+               .throw_error("Records could not be sorted according to ",
+                            ifelse(sort.by.slot,
+                                   paste0("slot = '", slot, "'"),
+                                   paste0("info_element = '", info_element, "'")))
+             })
+
+    ## return reordered object
+    return(object[ord])
+  }
+)
+
+# melt_RLum() -------------------------------------------------------------------------------
+#' @describeIn RLum.Analysis
+#' Melts [RLum.Analysis-class] objects into a flat data.frame to be used
+#' in combination with other packages such as `ggplot2`.
+#'
+#' @return
+#'
+#' **`melt_RLum`**
+#'
+#' Flat [data.frame] with `X`, `Y`, `TYPE`, `UID`
+#'
+#' @md
+#' @export
+setMethod(
+  f = "melt_RLum",
+  signature = "RLum.Analysis",
+  function(object) {
+    melt_RLum(object@records)
+  }
+)
+
+## view() -------------------------------------------------------------------
+#' @describeIn RLum.Analysis
+#'
+#' View method for [RLum.Analysis-class] objects
+#'
+#' @param ... other arguments that might be passed
+#'
+#' @keywords internal
+#'
+#' @md
+#' @export
+setMethod("view",
+          signature = "RLum.Analysis",
+          definition = function(object, ...) {
+
+    ## set title
+    name <- list(...)$title
+    if (is.null(name))
+      name <- deparse(substitute(object))
+
+    ## collect info lists from all records
+    info <- lapply(seq_along(object@records),
+                   function(x) c(aliquot = x, object@records[[x]]@info))
+    info <- rbindlist(info, fill = TRUE)
+
+    ## run view
+    .view(x = info, title = name)
+})
