@@ -3,7 +3,7 @@
 #' @description The function provides the analysis of cross-talk measurements on a
 #' FI lexsyg SMART reader using Al2O3:C chips
 #'
-#' @param object [RLum.Analysis-class] **(required)**:
+#' @param object [RLum.Analysis-class] or [list] (**required**):
 #' measurement input
 #'
 #' @param signal_integral [numeric] (*optional*):
@@ -19,14 +19,15 @@
 #' function [get_RLum]. To deactivate the automatic selection set the argument to `NULL`
 #'
 #' @param irradiation_time_correction [numeric] or [RLum.Results-class] (*optional*):
-#' information on the used irradiation time correction obtained by another experiments.
+#' information on the used irradiation time correction obtained by another
+#' experiment.
 #'
 #' @param method_control [list] (*optional*):
 #' optional parameters to control the calculation.
 #' See details for further explanations
 #'
 #' @param plot [logical] (*with default*):
-#' enable/disable plot output
+#' enable/disable the plot output.
 #'
 #' @param ... further arguments that can be passed to the plot output
 #'
@@ -92,71 +93,63 @@ analyse_Al2O3C_CrossTalk <- function(
   method_control = NULL,
   plot = TRUE,
   ...
-){
+) {
+  .set_function_name("analyse_Al2O3C_CrossTalk")
+  on.exit(.unset_function_name(), add = TRUE)
 
-  # Integrity check  ---------------------------------------------------------------------------
-
-  ##check input object
-  if(!all(unlist(lapply(object, function(x){is(x, "RLum.Analysis")})))){
-    stop("[analyse_Al2O3C_CrossTalk()] The elements in 'object' are not all of type 'RLum.Analysis'", call. = FALSE)
+  ## Integrity checks -------------------------------------------------------
 
+  .validate_class(object, c("RLum.Analysis", "list"))
+  .validate_not_empty(object, class(object)[1])
+  if (is.list(object)) {
+    lapply(object,
+           function(x) .validate_class(x, "RLum.Analysis",
+                                       name = "All elements of 'object'"))
   }
+  .validate_class(recordType, "character")
 
   ##TODO ... do more, push harder
   ##Accept the entire sequence ... including TL and extract
   ##Add sufficient unit tests
 
-  # Preparation ---------------------------------------------------------------------------------
+  ## Preparation ------------------------------------------------------------
+
   ##select curves based on the recordType selection; if not NULL
   if(!is.null(recordType)){
-    object <- get_RLum(object, recordType = recordType, drop = FALSE)
-
+    object <- suppressWarnings(get_RLum(object, recordType = recordType,
+                                        drop = FALSE))
+  }
+  if (is.null(object) || all(sapply(object, length) == 0)) {
+    .throw_error("'object' contains no records with recordType = '", recordType, "'")
   }
 
   #set method control
   method_control_settings <- list(
     fit.method = "EXP"
-
   )
 
   ##modify on request
   if(!is.null(method_control)){
-    if (!is.list(method_control)) {
-      stop("[analyse_Al2O3C_CrossTalk()] 'method_control' is expected ",
-           "to be a list", call. = FALSE)
-    }
+    .validate_class(method_control, "list")
     method_control_settings <- modifyList(x = method_control_settings, val = method_control)
-
   }
 
-
   ##set signal integral
+  max.signal_integral <- nrow(object[[1]][[1]][])
   if(is.null(signal_integral)){
-    signal_integral <- c(1:nrow(object[[1]][[1]][]))
+    signal_integral <- 1:max.signal_integral
 
   }else{
     ##check whether the input is valid, otherwise make it valid
-    if(min(signal_integral) < 1 | max(signal_integral) > nrow(object[[1]][[1]][])){
-      signal_integral <- c(1:nrow(object[[1]][[1]][]))
-      warning(
-        paste0(
-          "[analyse_Al2O3C_ITC()] Input for 'signal_integral' corrected to 1:", nrow(object[[1]][[1]][])
-        ),
-        call. = FALSE
-      )
+    if (min(signal_integral) < 1 || max(signal_integral) > max.signal_integral) {
+      signal_integral <- 1:max.signal_integral
+      .throw_warning("'signal_integral' corrected to 1:", max.signal_integral)
     }
-
   }
 
   ##check irradiation time correction
   if (!is.null(irradiation_time_correction)) {
-
-    if (!is.numeric(irradiation_time_correction) &&
-        !is(irradiation_time_correction, "RLum.Results")) {
-      stop("[analyse_Al2O3C_CrossTalk()] 'irradiation_time_correction' ",
-           "is expected to be a numeric value or an RLum.Results object",
-           call. = FALSE)
-    }
+    .validate_class(irradiation_time_correction, c("numeric", "RLum.Results"))
 
     if (is(irradiation_time_correction, "RLum.Results")) {
       if (irradiation_time_correction@originator == "analyse_Al2O3C_ITC") {
@@ -168,14 +161,10 @@ analyse_Al2O3C_CrossTalk <- function(
 
         }else{
           irradiation_time_correction <- c(irradiation_time_correction[[1]], irradiation_time_correction[[2]])
-
         }
-
       } else{
-        stop(
-          "[analyse_Al2O3C_CrossTalk()] The object provided for the argument 'irradiation_time_correction' was created by an unsupported function!",
-          call. = FALSE
-        )
+        .throw_error("The object provided for 'irradiation_time_correction' ",
+                     "was created by an unsupported function")
       }
     }
   }
@@ -215,7 +204,6 @@ analyse_Al2O3C_CrossTalk <- function(
     temp_df$DOSE_ERROR[id_zero] <- 0
 
     return(temp_df)
-
   })
 
   APPARENT_DOSE <- as.data.frame(data.table::rbindlist(lapply(1:length(object), function(x){
@@ -226,10 +214,8 @@ analyse_Al2O3C_CrossTalk <- function(
 
     }else{
       DOSE <- signal_table_list[[x]]$DOSE[2]
-
     }
 
-
     ##calculation
     temp <- (DOSE * signal_table_list[[x]]$NET_INTEGRAL[1])/signal_table_list[[x]]$NET_INTEGRAL[2]
 
@@ -237,13 +223,11 @@ analyse_Al2O3C_CrossTalk <- function(
       POSITION = signal_table_list[[x]]$POSITION[1],
       AD = mean(temp),
       AD_ERROR = sd(temp))
-
   })))
 
   ##add apparent dose to the information
   signal_table_list <- lapply(1:length(signal_table_list), function(x){
       cbind(signal_table_list[[x]], rep(APPARENT_DOSE[x,2:3], 2))
-
   })
 
   ##combine
@@ -288,7 +272,7 @@ analyse_Al2O3C_CrossTalk <- function(
     col.seq <- col.seq[["COLOUR"]][order(col.seq[["POSITION"]])]
 
     ##calculate model
-    fit <- lm(
+    fit <- stats::lm(
       formula = y ~ poly(x, 2, raw=TRUE),
       data = data.frame(y = APPARENT_DOSE$AD[order(APPARENT_DOSE$POSITION)], x = sort(APPARENT_DOSE$POSITION)))
 
@@ -338,7 +322,6 @@ analyse_Al2O3C_CrossTalk <- function(
              y = sin(step) * .85,
              labels = i)
         step <- step + arc.step
-
       }
 
       ##add center plot with position
@@ -388,7 +371,6 @@ analyse_Al2O3C_CrossTalk <- function(
         pos = 3,
         cex = 1.1
       )
-
     }
 
   # Output --------------------------------------------------------------------------------------
@@ -408,5 +390,4 @@ analyse_Al2O3C_CrossTalk <- function(
       call = sys.call()
     )
   )
-
 }
