@@ -1,5 +1,6 @@
-#' Calculate the cosmic dose rate
+#' @title Calculate the cosmic dose rate
 #'
+#' @description
 #' This function calculates the cosmic dose rate taking into account the soft-
 #' and hard-component of the cosmic ray flux and allows corrections for
 #' geomagnetic latitude, altitude above sea-level and geomagnetic field
@@ -11,7 +12,7 @@
 #' **Internal calculation steps**
 #'
 #' (1)
-#' Calculate total depth of all absorber in hg/cm^2 (1 hg/cm^2 = 100 g/cm^2)
+#' Calculate total depth of all absorber in hg/cm² (1 hg/cm² = 100 g/cm²)
 #'
 #' \deqn{absorber = depth_1*density_1 + depth_2*density_2 + ... + depth_n*density_n}
 #'
@@ -25,13 +26,13 @@
 #' (3)
 #' Calculate cosmic dose rate at sea-level and 55 deg. latitude
 #'
-#' a) If absorber is > 167 g/cm^2 (only hard-component; Allkofer et al.  1975):
+#' a) If absorber is > 167 g/cm² (only hard-component; Allkofer et al.  1975):
 #' apply equation given by Prescott & Hutton (1994) (c.f. Barbouti & Rastin
 #' 1983)
 #'
 #' \deqn{D0 = C/(((absorber+d)^\alpha+a)*(absober+H))*exp(-B*absorber)}
 #'
-#' b) If absorber is < 167 g/cm^2 (soft- and hard-component): derive D0 from
+#' b) If absorber is < 167 g/cm² (soft- and hard-component): derive D0 from
 #' Fig. 1 in Prescott & Hutton (1988).
 #'
 #'
@@ -78,14 +79,14 @@
 #' one sample of the same profile. This is done by providing more than one
 #' values for `depth` and only one for `density`. For example,
 #' `depth = c(1, 2, 3)` and `density = 1.7` will calculate the cosmic dose rate
-#' for three samples in 1, 2 and 3 m depth in a sediment of density 1.7 g/cm^3.
+#' for three samples in 1, 2 and 3 m depth in a sediment of density 1.7 g/cm³.
 #'
 #' @param depth [numeric] (**required**):
 #' depth of overburden (m). For more than one absorber use \cr
 #' `c(depth_1, depth_2, ..., depth_n)`
 #'
 #' @param density [numeric] (**required**):
-#' average overburden density (g/cm^3). For more than one absorber use \cr
+#' average overburden density (g/cm³). For more than one absorber use \cr
 #' `c(density_1, density_2, ..., density_n)`
 #'
 #' @param latitude [numeric] (**required**):
@@ -116,24 +117,24 @@
 #'
 #' @return
 #' Returns a terminal output. In addition an
-#' [RLum.Results-class]-object is returned containing the
+#' [RLum.Results-class] object is returned containing the
 #' following element:
 #'
 #' \item{summary}{[data.frame] summary of all relevant calculation results.}
 #' \item{args}{[list] used arguments}
 #' \item{call}{[call] the function call}
 #'
-#' The output should be accessed using the function [get_RLum]
+#' The output should be accessed using the function [get_RLum].
 #'
 #' @note
-#' Despite its universal use the equation to calculate the cosmic dose
+#' Despite its universal use, the equation to calculate the cosmic dose
 #' rate provided by Prescott & Hutton (1994) is falsely stated to be valid from
-#' the surface to 10^4 hg/cm^2 of standard rock. The original expression by
+#' the surface to 10^4 hg/cm² of standard rock. The original expression by
 #' Barbouti & Rastin (1983) only considers the muon flux (i.e. hard-component)
-#' and is by their own definition only valid for depths between 10-10^4
-#' hg/cm^2.
+#' and is, by their own definition, only valid for depths between 10-10^4
+#' hg/cm².
 #'
-#' Thus, for near-surface samples (i.e. for depths < 167 g/cm^2) the equation
+#' Thus, for near-surface samples (i.e. for depths < 167 g/cm²) the equation
 #' of Prescott & Hutton (1994) underestimates the total cosmic dose rate, as it
 #' neglects the influence of the soft-component of the cosmic ray flux. For
 #' samples at zero depth and at sea-level the underestimation can be as large
@@ -142,16 +143,16 @@
 #'
 #' \deqn{D = 0.21*exp(-0.070*absorber+0.0005*absorber^2)}
 #'
-#' which is valid for depths between 150-5000 g/cm^2. For shallower depths (<
-#' 150 g/cm^2) they provided a graph (Fig. 1) from which the dose rate can be
+#' which is valid for depths between 150-5000 g/cm². For shallower depths (<
+#' 150 g/cm²) they provided a graph (Fig. 1) from which the dose rate can be
 #' read.
 #'
 #' As a result, this function employs the equation of Prescott & Hutton (1994)
-#' only for depths > 167 g/cm^2, i.e. only for the hard-component of the cosmic
-#' ray flux. Cosmic dose rate values for depths < 167 g/cm^2 were obtained from
+#' only for depths > 167 g/cm², i.e. only for the hard-component of the cosmic
+#' ray flux. Cosmic dose rate values for depths < 167 g/cm² were obtained from
 #' the "AGE" program (Gruen 2009) and fitted with a 6-degree polynomial curve
 #' (and hence reproduces the graph shown in Prescott & Hutton 1988). However,
-#' these values assume an average overburden density of 2 g/cm^3.
+#' these values assume an average overburden density of 2 g/cm³.
 #'
 #' It is currently not possible to obtain more precise cosmic dose rate values
 #' for near-surface samples as there is no equation known to the author of this
@@ -248,39 +249,40 @@ calc_CosmicDoseRate<- function(
   error = 10,
   ...
 ) {
+  .set_function_name("calc_CosmicDoseRate")
+  on.exit(.unset_function_name(), add = TRUE)
 
-  ##============================================================================##
-  ## ... ARGUMENTS
-  ##============================================================================##
-  settings <- list(verbose = TRUE)
-  settings <- modifyList(settings, list(...))
+  ## Integrity checks -------------------------------------------------------
 
-  ##============================================================================##
-  ## CONSISTENCY CHECK OF INPUT DATA
-  ##============================================================================##
+  .validate_class(depth, "numeric")
+  .validate_class(density, "numeric")
+  .validate_class(latitude, "numeric")
+  .validate_class(longitude, "numeric")
+  .validate_class(altitude, "numeric")
 
   if(any(depth < 0) || any(density < 0)) {
-    stop("[calc_CosmicDoseRate()] No negative values allowed for ",
-         "depth and density", call. = FALSE)
+    .throw_error("No negative values allowed for 'depth' and 'density'")
   }
 
   if(corr.fieldChanges == TRUE) {
     if(is.na(est.age) == TRUE) {
-      stop("[calc_CosmicDoseRate()] Correction for geomagnetic field ",
-           "changes requires an age estimate.", call. = FALSE)
+      .throw_error("Correction for geomagnetic field ",
+                   "changes requires an age estimate")
     }
     if(est.age > 80) {
       cat("\nCAUTION: No geomagnetic field change correction for samples",
-          "older >80 ka possible!")
+          "older than 80 ka possible, 'corr.fieldChanges' set to FALSE")
       corr.fieldChanges<- FALSE
     }
   }
 
   if(length(density) > length(depth)) {
-    stop("\nIf you provide more than one value for density please",
-         " provide an equal number of values for depth.", call. = FALSE)
+    .throw_error("If you provide more than one value for density, please ",
+                 "provide an equal number of values for depth")
   }
 
+  settings <- list(verbose = TRUE)
+  settings <- modifyList(settings, list(...))
 
   ##============================================================================##
   ## CALCULATIONS
@@ -341,28 +343,20 @@ calc_CosmicDoseRate<- function(
 
   for(i in 1:length(hgcm)) {
 
-
     # calculate cosmic dose rate at sea-level for geomagnetic latitude 55 degrees
+    d0 <- (C / ((((hgcm[i] + d)^alpha) + a) * (hgcm[i] + H))) * exp(-B * hgcm[i])
 
-    if(hgcm[i]*100 >= 167) {
+    temp.hgcm <- hgcm[i] * 100
+    if (temp.hgcm < 167) {
+      d0.ph <- d0
 
-      d0<- (C/((((hgcm[i]+d)^alpha)+a)*(hgcm[i]+H)))*exp(-B*hgcm[i])
-
-    }
-    if(hgcm[i]*100 < 167) {
-
-      temp.hgcm<- hgcm[i]*100
-      d0.ph<- (C/((((hgcm[i]+d)^alpha)+a)*(hgcm[i]+H)))*exp(-B*hgcm[i])
-
-      if(hgcm[i]*100 < 40) {
+      if (temp.hgcm < 40) {
         d0<- -6*10^-8*temp.hgcm^3+2*10^-5*temp.hgcm^2-0.0025*temp.hgcm+0.2969
       }
       else {
         d0<- 2*10^-6*temp.hgcm^2-0.0008*temp.hgcm+0.2535
       }
-      if(d0.ph > d0) {
-        d0<- d0.ph
-      }
+      d0 <- max(d0, d0.ph)
     }
     # Calculate geomagnetic latitude
     gml.temp<- 0.203*cos((pi/180)*latitude)*
@@ -382,13 +376,11 @@ calc_CosmicDoseRate<- function(
     else { # Linear fit
 
       F_ph<- -0.0001*gml + 0.2347
-
     }
 
     if(gml < 34) { # Polynomial fit
 
       J_ph<- 5*10^-6*gml^3-5*10^-5*gml^2+0.0026*gml+0.5177
-
     }
     else { # Linear fit
       J_ph<- 0.0005*gml + 0.7388
@@ -397,12 +389,10 @@ calc_CosmicDoseRate<- function(
     if(gml < 36) { # Polynomial fit
 
       H_ph<- -3*10^-6*gml^3-5*10^-5*gml^2-0.0031*gml+4.398
-
     }
     else { # Linear fit
 
       H_ph<- 0.0002*gml + 4.0914
-
     }
 
     # Apply correction for geomagnetic latitude and altitude according to
@@ -433,43 +423,12 @@ calc_CosmicDoseRate<- function(
         corr.matrix[6,]<- c(1.03, 1.03, 1.02, 1.01, 1.00, 1.00)
         corr.matrix[7,]<- c(1.02, 1.02, 1.02, 1.01, 1.00, 1.00)
 
-        # Find corresponding correction factor for given geomagnetic latitude
-
-        # determine column
-        if(gml <= 5) { corr.c<- 1 }
-        if(5 < gml) {
-          if(gml <= 15) { corr.c<- 2 }
-        }
-        if(15 < gml){
-          if(gml <= 25) { corr.c<- 3 }
-        }
-        if(25 < gml){
-          if(gml <= 32.5) { corr.c<- 4 }
-        }
-        if(32.5 < gml){
-          if(gml <= 35) { corr.c<- 5 }
-        }
-
-        # find row
-        if(est.age <= 5) { corr.fac<- corr.matrix[1,corr.c] }
-        if(5 < est.age) {
-          if(est.age <= 10) { corr.fac<- corr.matrix[2,corr.c] }
-        }
-        if(10 < est.age){
-          if(est.age <= 15) { corr.fac<- corr.matrix[3,corr.c] }
-        }
-        if(15 < est.age){
-          if(est.age <= 20) { corr.fac<- corr.matrix[4,corr.c] }
-        }
-        if(20 < est.age){
-          if(est.age <= 35) { corr.fac<- corr.matrix[5,corr.c] }
-        }
-        if(35 < est.age){
-          if(est.age <= 50) { corr.fac<- corr.matrix[6,corr.c] }
-        }
-        if(50 < est.age){
-          if(est.age <= 80) { corr.fac<- corr.matrix[7,corr.c] }
-        }
+        ## Find the correction factor for the given geomagnetic latitude
+        row.idx <- cut(est.age, c(0, 5, 10, 15, 20, 35, 50, 80),
+                       labels = FALSE)
+        col.idx <- cut(gml, c(0, 5, 15, 25, 32.5, 35, Inf),
+                       labels = FALSE)
+        corr.fac <- corr.matrix[row.idx, col.idx]
 
         # Find altitude factor via fitted function 2-degree polynomial
         # This factor is only available for positive altitudes
@@ -479,11 +438,8 @@ calc_CosmicDoseRate<- function(
 
           # Combine geomagnetic latitude correction with altitude
           # correction (figure caption of Fig. 1 in Precott and Hutton (1994))
-
-
           diff.one<- corr.fac - 1
           corr.fac<- corr.fac + diff.one * alt.fac
-
         }
 
         # Final correction of cosmic dose rate
@@ -495,7 +451,7 @@ calc_CosmicDoseRate<- function(
 
       } else {
         if (settings$verbose)
-          cat(paste("\n No geomagnetic field change correction necessary for geomagnetic latitude >35 degrees!"))
+          cat("\n No geomagnetic field change correction necessary for geomagnetic latitude >35 degrees!")
       }
     }
 
@@ -578,15 +534,6 @@ calc_CosmicDoseRate<- function(
 
     summary<- data.frame(cbind(temp1,temp2))
 
-    newRLumResults.calc_CosmicDoseRate <- set_RLum(
-      class = "RLum.Results",
-      data = list(summary=summary,
-                  args=args,
-                  call=call))
-
-    # Return values
-    invisible(newRLumResults.calc_CosmicDoseRate)
-
   } else {
 
     #terminal output
@@ -605,15 +552,12 @@ calc_CosmicDoseRate<- function(
     colnames(profile.results)<- c("depth","d0","dc","dc_err")
 
     summary<- data.frame(cbind(profile.results,add.info))
+  }
 
-    newRLumResults.calc_CosmicDoseRate <- set_RLum(
+  newRLumResults.calc_CosmicDoseRate <- set_RLum(
       class = "RLum.Results",
       data = list(summary=summary,
                   args=args,
                   call=call))
-
-    # Return values
-    invisible(newRLumResults.calc_CosmicDoseRate)
-
-  }
+  invisible(newRLumResults.calc_CosmicDoseRate)
 }
