@@ -26,7 +26,7 @@
 #' number of commits returned (defaults to 5).
 #'
 #' @param verbose [logical] (*with default*):
-#' print the output to the console (defaults to `TRUE`).
+#' enable/disable output to the terminal.
 #'
 #' @author Christoph Burow, University of Cologne (Germany)
 #'
@@ -68,17 +68,19 @@ NULL
 #' @export
 github_commits <- function(user = "r-lum", repo = "luminescence",
                            branch = "master", n = 5) {
+  .set_function_name("github_commits")
+  on.exit(.unset_function_name(), add = TRUE)
 
   # fetch available branches and check if provided branch exists
   branches <- github_branches(user, repo)
   if (!any(grepl(branch, branches$BRANCH)))
-    stop("Branch ", branch, " does not exist.", call. = FALSE)
+    .throw_error("Branch '", branch, "' does not exist")
 
   # build URL and retrieve content
   sha <- branches$SHA[grep(paste0("^", branch, "$"), branches$BRANCH)]
   url <- paste0("https://api.github.com/repos/", user, "/", repo, "/commits?",
                 "per_page=", n, "&sha=", sha)
-  content <- github_getContent(url)
+  content <- .github_getContent(url)
 
   # format output as data.frame
   output <- do.call(rbind, lapply(content, function(x) {
@@ -113,10 +115,15 @@ github_commits <- function(user = "r-lum", repo = "luminescence",
 #' @md
 #' @export
 github_branches <- function(user = "r-lum", repo = "luminescence") {
+  .set_function_name("github_branches")
+  on.exit(.unset_function_name(), add = TRUE)
+
+  .validate_class(user, "character")
+  .validate_class(repo, "character")
 
   # build URL and retrieve content
   url <- paste0("https://api.github.com/repos/", user, "/", repo, "/branches")
-  content <- github_getContent(url)
+  content <- .github_getContent(url)
 
   # extract relevant information from server response
   branches <- sapply(content, function(x) x$name)
@@ -158,10 +165,15 @@ github_branches <- function(user = "r-lum", repo = "luminescence") {
 #' @md
 #' @export
 github_issues <- function(user = "r-lum", repo = "luminescence", verbose = TRUE) {
+  .set_function_name("github_issues")
+  on.exit(.unset_function_name(), add = TRUE)
+
+  .validate_class(user, "character")
+  .validate_class(repo, "character")
 
   # build URL and retrieve content
   url <- paste0("https://api.github.com/repos/", user,"/", repo, "/issues")
-  content <- github_getContent(url)
+  content <- .github_getContent(url)
 
   # format output as nested list
   issues <- lapply(content, function(x) {
@@ -177,7 +189,7 @@ github_issues <- function(user = "r-lum", repo = "luminescence", verbose = TRUE)
       MILESTONE = x$milestone$title)
   })
 
-  # custom printing of the the issues-list as print.list produces unreadable
+  # custom printing of the issues list, as print.list produces unreadable
   # console output
   if (verbose) {
     tmp <- lapply(issues, function(x) {
@@ -200,7 +212,6 @@ github_issues <- function(user = "r-lum", repo = "luminescence", verbose = TRUE)
                  'milestone: "', x$MILESTONE, '"', "\n",
                  "description: >\n", DESCRIPTION,
                  "\n\n\n"))
-
     })
   }
   # return invisible as we explicitly print the output
@@ -208,16 +219,17 @@ github_issues <- function(user = "r-lum", repo = "luminescence", verbose = TRUE)
 }
 
 
-
 # HELPER ------------------------------------------------------------------
 
 # This function queries the URL, checks the server response and returns
 # the content.
-github_getContent <- function(url) {
-  response <- GET(url, accept_json())
-  if (status_code(response) != 200)
-    stop("Contacting ", url, " had status code ", status_code(response),
-         call. = FALSE)
-  content <- content(response)
+.github_getContent <- function(url) {
+  response <- httr::GET(url, httr::accept_json())
+  # nocov start
+  if (httr::status_code(response) != 200)
+    .throw_error("Contacting ", url, " returned status code ",
+                 httr::status_code(response))
+  # nocov end
+  content <- httr::content(response)
   return(content)
 }
