@@ -51,12 +51,13 @@
 #' in fewer curves plotted as specified. This parameter has only
 #' an effect for  n > 2.
 #'
-#' @param curve.transformation [character] (*optional*):
+#' @param curve.transformation [character] (*with default*):
 #' allows transforming CW-OSL and CW-IRSL curves to pseudo-LM curves via
 #' transformation functions. Allowed values are: `CW2pLM`, `CW2pLMi`,
-#' `CW2pHMi` and `CW2pPMi`. See details.
+#' `CW2pHMi` and `CW2pPMi`, see details. If set to `None` (default), no
+#' transformation is applied.
 #'
-#' @param plot.single [logical] (*with default*):
+#' @param plot_singlePanels [logical] (*with default*):
 #' global par settings are considered, normally this should end in one plot per page
 #'
 #' @param ... further arguments and graphical parameters will be passed to
@@ -78,7 +79,7 @@
 #' way you might expect them to work. This function was designed to serve as an overview
 #' plot, if you want to have more control, extract the objects and plot them individually.
 #'
-#' @section Function version: 0.3.15
+#' @section Function version: 0.3.16
 #'
 #' @author
 #' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
@@ -125,26 +126,21 @@ plot_RLum.Analysis <- function(
   abline = NULL,
   combine = FALSE,
   records_max = NULL,
-  curve.transformation,
-  plot.single = FALSE,
+  curve.transformation = "None",
+  plot_singlePanels = FALSE,
   ...
 ) {
   .set_function_name("plot_RLum.Analysis")
   on.exit(.unset_function_name(), add = TRUE)
 
-  # Integrity check ----------------------------------------------------------------------------
+  ## Integrity checks -------------------------------------------------------
 
-  ##check if object is of class RLum.Analysis (lists are handled via plot_RLum())
-  if (!is(object, "RLum.Analysis"))
-    .throw_error("Input object is not of type 'RLum.Analysis'")
-
-  # Make selection if wanted  -------------------------------------------------------------------
+  .validate_class(object, "RLum.Analysis")
 
   if(!is.null(subset)){
     ##check whether the user set the drop option and remove it, as we cannot work with it
     subset <- subset[!sapply(names(subset), function(x){"drop" %in% x})]
     object <- do.call(get_RLum, c(object = object, subset, drop = FALSE))
-
   }
 
   # Deal with additional arguments.  ------------------------------------------------------------
@@ -175,95 +171,90 @@ plot_RLum.Analysis <- function(
 
   plot.settings <- modifyList(x = plot.settings, val = list(...), keep.null = TRUE)
 
+  ## deprecated argument
+  if ("plot.single" %in% names(list(...))) {
+    plot_singlePanels <- list(...)$plot.single
+    .throw_warning("'plot.single' is deprecated, use 'plot_singlePanels' ",
+                   "instead")
+  }
+
   ##try to find optimal parameters, this is however, a little bit stupid, but
   ##better than without any presetting
-  if(combine)
+  .validate_logical_scalar(combine)
+  if (combine && length(object@records) <= 1) {
+    combine <- FALSE
+    .throw_warning("'combine' can't be used with fewer than two curves, ",
+                   "reset to FALSE")
+  }
+  if (combine) {
+    sapply(object@records, function(x) {
+      if (!inherits(x, "RLum.Data.Curve")) {
+        .throw_error("'combine' is valid only for 'RLum.Data.Curve' objects")
+      }
+    })
     n.plots <- length(unique(as.character(structure_RLum(object)$recordType)))
+  }
   else
-    n.plots <- length_RLum(object)
+    n.plots <- max(length_RLum(object), 1)
 
-  if (!missing(nrows)) .validate_positive_scalar(nrows)
-  if (!missing(ncols)) .validate_positive_scalar(ncols)
+  .validate_positive_scalar(nrows)
+  .validate_positive_scalar(ncols)
 
   ## set appropriate values for nrows and ncols if not both specified
-  if (missing(ncols) | missing(nrows)) {
-    if (missing(ncols) & !missing(nrows)) {
-      if (n.plots  == 1) {
+  if (missing(nrows) || missing(ncols)) {
+    if (n.plots == 1) {
+      if (missing(nrows))
+        nrows <- 1
+      if (missing(ncols))
         ncols <- 1
 
-      } else{
+    } else { # n.plots > 1
+      if (missing(ncols)) {
         ncols <- 2
       }
-    }
-    else if (!missing(ncols) & missing(nrows)) {
-      if (n.plots  == 1) {
-        nrows <- 1
-
-      }
-      else if (n.plots  > 1 & n.plots <= 4) {
-        nrows <- 2
-
-      } else{
-        nrows <- 3
-      }
-
-    } else{
-
-      if (n.plots  == 1) {
-        nrows <- 1
-        ncols <- 1
-
-      }
-      else if (n.plots  > 1 & n.plots  <= 2) {
-        nrows <- 1
-        ncols <- 2
-
-      } else if (n.plots  > 2 & n.plots <= 4) {
-        nrows <- 2
-        ncols <- 2
-
-      }
-      else{
-        nrows <- 3
-        ncols <- 2
-
+      if (missing(nrows)) {
+        if (n.plots <= 4) {
+          nrows <- ceiling(n.plots / 2)
+        } else {
+          nrows <- 3
+        }
       }
     }
   }
 
+  curve.transformation <- .validate_args(curve.transformation,
+                                         c("CW2pLM", "CW2pLMi",
+                                           "CW2pHMi", "CW2pPMi", "None"))
+  ## complete the function name
+  if (curve.transformation != "None") {
+    curve.transformation <- paste0("convert_", curve.transformation)
+  }
 
   # Plotting ------------------------------------------------------------------
   ##+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   ##(1) NORMAL (combine == FALSE)
   ##+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-  if(!combine || length(object@records) == 1){
 
-    ##show warning message
-    if(combine & length(object@records) == 1){
-      .throw_warning("Nothing to combine, object contains a single curve")
-    }
+  if (!combine) {
 
     ##grep RLum.Data.Curve or RLum.Data.Spectrum objects
-    temp <- lapply(1:length(object@records), function(x){
-      if(is(object@records[[x]], "RLum.Data.Curve") ||
-         is(object@records[[x]], "RLum.Data.Spectrum")){
-
-        object@records[[x]]
-
+    temp <- lapply(object@records, function(x) {
+      if (inherits(x, "RLum.Data.Curve") ||
+          inherits(x, "RLum.Data.Spectrum")) {
+        x
       }})
 
     ##calculate number of pages for mtext
-    if (length(temp) %% (nrows * ncols) > 0) {
+    if (length(temp) == 0 || length(temp) %% (nrows * ncols) > 0) {
       n.pages <- round(length(temp) / (nrows * ncols), digits = 0) + 1
 
     } else{
       n.pages <- length(temp) / (nrows * ncols)
-
     }
 
     ##set par
     par.default <- par("mfrow")
-    if(!plot.single) {
+    if (!plot_singlePanels) {
       par(mfrow = c(nrows, ncols))
       on.exit(par(mfrow = par.default), add = TRUE)
     }
@@ -276,20 +267,16 @@ plot_RLum.Analysis <- function(
 
                                   if(is(plot.settings[[x]], "list")){
                                     rep_len(plot.settings[[x]], length.out = length(temp))
-
                                   }else{
                                    rep_len(list(plot.settings[[x]]), length.out = length(temp))
-
                                   }
 
                                 }else{
                                   rep_len(plot.settings[[x]], length.out = length(temp))
-
                                 }
 
                               } else{
                                 plot.settings[[x]]
-
                               }
                             })
 
@@ -298,34 +285,19 @@ plot_RLum.Analysis <- function(
       abline.names <- rep_len(names(abline), length.out = length(temp))
       abline <- rep_len(abline, length.out = length(temp))
       names(abline) <- abline.names
-
     }
 
     ##apply curve transformation
-    for(i in 1:length(temp)){
+    for (i in seq_along(temp)) {
 
-      if(is(temp[[i]], "RLum.Data.Curve") == TRUE){
+      if (inherits(temp[[i]], "RLum.Data.Curve")) {
 
         ##set curve transformation if wanted
-        if((grepl("IRSL", temp[[i]]@recordType) | grepl("OSL", temp[[i]]@recordType)) &
-           !missing(curve.transformation)){
+        if (grepl("IRSL|OSL", temp[[i]]@recordType) &&
+            curve.transformation != "None") {
 
-          if(curve.transformation=="CW2pLM"){
-            temp[[i]] <- CW2pLM(temp[[i]])
-
-          }else if(curve.transformation=="CW2pLMi"){
-            temp[[i]] <- CW2pLMi(temp[[i]])
-
-          }else if(curve.transformation=="CW2pHMi"){
-            temp[[i]]<- CW2pHMi(temp[[i]])
-
-          }else if(curve.transformation=="CW2pPMi"){
-            temp[[i]] <- CW2pPMi(temp[[i]])
-
-          }else{
-            .throw_warning("Function for 'curve.transformation' is unknown, ",
-                           "no transformation performed")
-          }
+          ## get the actual function from the parameter value and apply it
+          temp[[i]] <- get(curve.transformation)(temp[[i]])
         }
 
         ##check plot settings and adjust
@@ -363,7 +335,6 @@ plot_RLum.Analysis <- function(
 
         }else{
           ylim.set <- plot.settings$ylim[[i]]
-
         }
 
         ##col
@@ -371,15 +342,12 @@ plot_RLum.Analysis <- function(
           col <- plot.settings$col[i]
 
         } else{
+          col <- "black"
           if (grepl("IRSL", temp[[i]]@recordType)) {
             col <- "red"
-          } else
-            if (grepl("OSL", temp[[i]]@recordType)) {
-              col <- "blue"
-            } else
-            {
-              col <- "black"
-            }
+          } else if (grepl("OSL", temp[[i]]@recordType)) {
+            col <- "blue"
+          }
         }
 
         ##main
@@ -416,6 +384,7 @@ plot_RLum.Analysis <- function(
               norm = plot.settings$norm,
               pch = plot.settings$pch[[i]],
               cex = plot.settings$cex[[i]],
+              legend.col = plot.settings$legend.col[[i]],
               smooth = plot.settings$smooth[[i]]
             ),
             list(...)
@@ -430,7 +399,6 @@ plot_RLum.Analysis <- function(
         ##add abline
         if(!is.null(abline[[i]])){
           do.call(what = "abline", args = abline[i])
-
         }
 
       } else if(inherits(temp[[i]], "RLum.Data.Spectrum")) {
@@ -443,7 +411,6 @@ plot_RLum.Analysis <- function(
             par.local = FALSE,
             main = if(!is.null(plot.settings$main)) plot.settings$main else temp[[i]]@recordType
         ), args))
-
       }
 
     }#end for loop
@@ -454,34 +421,21 @@ plot_RLum.Analysis <- function(
     ##(2) NORMAL (combine == TRUE)
     ##++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     ##(1) check RLum objects in the set
-    object.list <- object@records
-
-    sapply(object.list, function(o){
-      if(!inherits(o, "RLum.Data.Curve")){
-        .throw_error("Using 'combine' is limited to 'RLum.Data.Curve' objects")
-      }
-    })
 
     ##account for different curve types, combine similar
     temp.object.structure  <- structure_RLum(object)
     temp.recordType <- as.character(unique(temp.object.structure$recordType))
 
     ##change graphic settings
-    if(!plot.single){
-      par.default <- par()[c("cex", "mfrow")]
-
+    par.default <- par()[c("cex", "mfrow")]
+    if (!plot_singlePanels) {
       if(!missing(ncols) & !missing(nrows)){
         par(mfrow = c(nrows, ncols))
       }
-
-      ##this 2nd par request is needed as setting mfrow resets the par settings ... this might
-      ##not be wanted
-      par(cex = plot.settings$cex[1])
-
-    }else{
-      par.default <- par()[c("cex")]
-      par(cex = plot.settings$cex)
     }
+    ## this 2nd par request is needed as setting mfrow resets the par
+    ## settings ... this might not be wanted
+    par(cex = plot.settings$cex)
 
     ##expand plot settings list
     ##expand list
@@ -496,7 +450,6 @@ plot_RLum.Analysis <- function(
 
       } else{
         plot.settings[[x]]
-
       }
     })
 
@@ -505,7 +458,6 @@ plot_RLum.Analysis <- function(
       abline.names <- rep_len(names(abline), length.out = length(temp.recordType))
       abline <- rep_len(abline, length.out = length(temp.recordType))
       names(abline) <- abline.names
-
     }
 
 
@@ -528,57 +480,26 @@ plot_RLum.Analysis <- function(
       if(!is.null(records_max) && records_max[1] > 2){
         records_show <- ceiling(seq(1,length(object.list), length.out = records_max))
         object.list[(1:length(object.list))[-records_show]] <- NULL
-
-      }
-
-      ##prevent problems for non set argument
-      if (missing(curve.transformation)) {
-        curve.transformation <- "None"
       }
 
       ##transform values to data.frame and norm values
       temp.data.list <- lapply(1:length(object.list), function(x) {
-        ##set curve transformation if wanted
 
-        if (grepl("IRSL", object.list[[x]]@recordType) |
-            grepl("OSL", object.list[[x]]@recordType)) {
-          if (curve.transformation == "CW2pLM") {
-            object.list[[x]] <- CW2pLM(object.list[[x]])
+        ## set curve transformation if wanted
+        if (grepl("IRSL|OSL", object.list[[x]]@recordType) &&
+            curve.transformation != "None") {
 
-          }else if (curve.transformation == "CW2pLMi") {
-            object.list[[x]] <- CW2pLMi(object.list[[x]])
-
-          }else if (curve.transformation == "CW2pHMi") {
-            object.list[[x]] <- CW2pHMi(object.list[[x]])
-
-          }else if (curve.transformation == "CW2pPMi") {
-            object.list[[x]] <- CW2pPMi(object.list[[x]])
-          }
+          ## get the actual function from the parameter value and apply it
+          object.list[[x]] <- get(curve.transformation)(object.list[[x]])
         }
 
         temp.data <- as(object.list[[x]], "data.frame")
 
-        ##normalise curves if argument has been set
-        if(plot.settings$norm[[k]][1] %in% c('max', 'last', 'huot') || plot.settings$norm[[k]][1] == TRUE){
-          if (plot.settings$norm[[k]] == "max" || plot.settings$norm[[k]] == TRUE) {
-             temp.data[[2]] <-  temp.data[[2]] / max(temp.data[[2]])
-
-          } else if (plot.settings$norm[[k]] == "last") {
-            temp.data[[2]] <-  temp.data[[2]] / temp.data[[2]][length(temp.data[[2]])]
-
-
-          } else if (plot.settings$norm[[k]] == "huot") {
-            bg <- median(temp.data[[2]][floor(nrow(temp.data)*0.8):nrow(temp.data)])
-            temp.data[[2]] <-  (temp.data[[2]] - bg) / max(temp.data[[2]] - bg)
-
-          }
-
-          ##check for Inf and NA
-          if(any(is.infinite(temp.data[[2]])) || anyNA(temp.data[[2]])){
-            temp.data[[2]][is.infinite(temp.data[[2]]) | is.na(temp.data[[2]])] <- 0
-            .throw_warning("Normalisation led to Inf or NaN values, ",
-                           "values replaced by 0")
-          }
+        ## curve normalisation
+        if (plot.settings$norm[[k]][1] == TRUE ||
+            plot.settings$norm[[k]][1] %in% c("max", "last", "huot")) {
+          temp.data[[2]] <- .normalise_curve(temp.data[[2]],
+                                             plot.settings$norm[[k]])
         }
 
         return(temp.data)
@@ -597,14 +518,10 @@ plot_RLum.Analysis <- function(
       xlab <- if(!is.null(plot.settings$xlab[[k]])){
         plot.settings$xlab[[k]]
       }else{
-        switch(temp.recordType[[k]],
-               "TL" = "Temperature [\u00B0C]",
-               "IRSL" = "Time [s]",
-               "OSL" = "Time [s]",
-               "RF" = "Time [s]",
-               "RBR" = "Time [s]",
-               "LM-OSL" = "Time [s]"
-        )
+        if (temp.recordType[[k]] == "TL")
+          "Temperature [\u00B0C]"
+        else
+          "Time [s]"
       }
 
       ##ylab
@@ -630,7 +547,6 @@ plot_RLum.Analysis <- function(
         range(unlist(lapply(X = temp.data.list, FUN = function(x){
           range(x[,2])
         })))
-
       }
 
       if (grepl("y", plot.settings$log[[k]], ignore.case = TRUE))
@@ -642,31 +558,23 @@ plot_RLum.Analysis <- function(
 
       }else{
         col <- get("col", pos = .LuminescenceEnv)
-
       }
 
       ##if length of provided colours is < the number of objects, just one colour is supported
       if (length(col) < length(object.list)) {
         col <- rep_len(col, length(object.list))
-
       }
 
       ##lty
+      lty <- plot.settings$lty[[k]]
       if (length(plot.settings$lty[[k]]) < length(object.list)) {
         lty <- rep(plot.settings$lty[[k]], times = length(object.list))
-
-      }else{
-        lty <- plot.settings$lty[[k]]
-
       }
 
       ##pch
+      pch <- plot.settings$pch[[k]]
       if (length(plot.settings$pch[[k]]) < length(object.list)) {
         pch <- rep(plot.settings$pch[[k]], times = length(object.list))
-
-      }else{
-        pch <- plot.settings$pch[[k]]
-
       }
 
       ##legend.text
@@ -679,19 +587,11 @@ plot_RLum.Analysis <- function(
 
         } else {
           paste("Curve", 1:length(object.list))
-
         }
-
       }
 
       ##legend.col
-      legend.col <- if(!is.null(plot.settings$legend.col[[k]])){
-        plot.settings$legend.col[[k]]
-
-      }else{
-        NULL
-
-      }
+      legend.col <- plot.settings$legend.col[[k]]
 
       ##legend.pos
       legend.pos <- if(!is.null(plot.settings$legend.pos[[k]])){
@@ -699,7 +599,6 @@ plot_RLum.Analysis <- function(
 
       }else{
         "topright"
-
       }
 
       if (legend.pos == "outside") {
@@ -723,15 +622,15 @@ plot_RLum.Analysis <- function(
       ## ...?Why using matplot is a bad idea: The channel resolution might be different
       for (n in 1:length(temp.data.list)) {
 
-
         ##smooth
         ##Why here again ... because the call differs from the one before, where the argument
         ##is passed to plot_RLum.Data.Curve()
         if(plot.settings$smooth[[k]]){
 
           k_factor <- ceiling(length(temp.data.list[[n]][, 2])/100)
-          temp.data.list[[n]][, 2] <- zoo::rollmean(temp.data.list[[n]][, 2],
-                                            k = k_factor, fill = NA)
+          temp.data.list[[n]][, 2] <- .smoothing(temp.data.list[[n]][, 2],
+                                                 k = k_factor, fill = NA,
+                                                 align = "center")
         }
 
         ##remove 0 values if plotted on a log-scale
@@ -750,7 +649,6 @@ plot_RLum.Analysis <- function(
             lty = lty[n],
             lwd = plot.settings$lwd[[k]]
           )
-
         }
 
         ##add points if requested
@@ -766,7 +664,6 @@ plot_RLum.Analysis <- function(
       ##add abline
       if(!is.null(abline[[k]])){
         do.call(what = "abline", args = abline[k])
-
       }
 
       ##mtext
@@ -778,16 +675,14 @@ plot_RLum.Analysis <- function(
         par(xpd = TRUE)
 
         # determine legend position on log(y) scale
+        ypos <- par()$usr[4]
         if (grepl("y", plot.settings$log[[k]], ignore.case = TRUE))
           ypos <- 10^par()$usr[4]
-        else
-          ypos <- par()$usr[4]
 
         # determine position on log(x) scale
+        xpos <- par()$usr[2]
         if (grepl("x", plot.settings$log[[k]], ignore.case = TRUE))
           xpos <- 10^par()$usr[2]
-        else
-          xpos <- par()$usr[2]
       }
 
       ##legend
@@ -816,9 +711,7 @@ plot_RLum.Analysis <- function(
     ##reset graphic settings
     if (exists("par.default.outside")) {
       par(par.default.outside)
-      rm(par.default.outside)
     }
     par(par.default)
-    rm(par.default)
   }
 }
