@@ -22,9 +22,10 @@
 #' limit frame range, e.g. select first 100 frames by `frame.range = c(1,100)`
 #'
 #' @param txtProgressBar [logical] (*with default*):
-#' enables or disables [txtProgressBar].
+#' enable/disable the progress bar. Ignored if `verbose = FALSE`.
 #'
-#' @param verbose [logical] (*with default*): enables or disables verbose mode
+#' @param verbose [logical] (*with default*): enable/disable output to the
+#' terminal.
 #'
 #' @param ... not used, for compatibility reasons only
 #'
@@ -114,16 +115,16 @@ read_SPE2R <- function(
   .set_function_name("read_SPE2R")
   on.exit(.unset_function_name(), add = TRUE)
 
-  # Consistency check -------------------------------------------------------
+  ## Integrity checks -------------------------------------------------------
 
+  .validate_class(file, "character")
+  .validate_not_empty(file)
   valid.output.object <- c("RLum.Data.Image", "RLum.Data.Spectrum", "matrix")
-  if (!output.object %in% valid.output.object) {
-    .throw_error("'output.object' not supported, valid options are ",
-                 paste(valid.output.object, collapse = ", "))
-  }
+  .validate_args(output.object, valid.output.object)
 
   ##check if file exists
   if(!file.exists(file)){
+    failed <- TRUE
 
     ## check if the file is an URL ... you never know
     if (grepl(pattern = "^https?://", x = file)) {
@@ -139,18 +140,16 @@ read_SPE2R <- function(
         file_link <- tempfile("read_SPE2R_FILE", fileext = ".SPE")
         download.file(file, destfile = file_link, quiet = !verbose, mode = "wb")
         file <- file_link
-
+        failed <- FALSE
       }else{
         if (verbose) cat("FAILED\n")
-        message("[read_SPE2R()] Error: File does not exist, NULL returned")
-        return(NULL)
       }
-
-    }else{
-      message("[read_SPE2R()] Error: File does not exist, NULL returned")
-      return(NULL)
     }
 
+    if (failed) {
+      .throw_message("File does not exist, NULL returned")
+      return(NULL)
+    }
   }
 
   ##check file extension
@@ -161,10 +160,20 @@ read_SPE2R <- function(
   }}
 
 
+  if (!verbose)
+    txtProgressBar <- FALSE
+
   # Open Connection ---------------------------------------------------------
 
   con <- file(file, "rb")
 
+  if (verbose) {
+    cat("\n[read_SPE2R()] Importing ...")
+    cat("\n path: ", dirname(file))
+    cat("\n file: ", .shorten_filename(basename(file)))
+    cat("\n")
+  }
+
   # read header -------------------------------------------------------------
 
   temp <- readBin(con, what="int", 2, size=2, endian="little", signed = TRUE)
@@ -306,39 +315,17 @@ read_SPE2R <- function(
   # read count value data ---------------------------------------------------
   ##set functions
 
-  if(datatype  == 0){
-    read.data <- function(n.counts){
-      readBin(con, what="double", n.counts, size=4, endian="little")
-    }
-
-  }else if(datatype == 1){
-
-    read.data <- function(n.counts){
-      readBin(con, what="integer", n.counts, size=4, endian="little", signed = TRUE)
-    }
-
-  }else if(datatype == 2){
-
-    read.data <- function(n.counts){
-      readBin(con, what="integer", n.counts, size=2, endian="little", signed = TRUE)
-    }
-
-  }else if(datatype == 3){
-    read.data <- function(n.counts){
-      readBin(con, what="int", n.counts, size=2, endian="little", signed = FALSE)
-
-    }
-
-  }else if(datatype == 8){
-
-    read.data <- function(n.counts){
-      readBin(con, what="integer", n.counts, size=4, endian="little", signed = FALSE)
-    }
-
-  }else{
-    .throw_error("Unknown 'datatype'")
+  ## define the reading function according to the datatype
+  if (!datatype %in% c(0, 1, 2, 3, 8)) {
+    .throw_error("Unknown 'datatype'") # nocov
+  }
+  what <- if (datatype == 0) "double" else "integer"
+  size <- if (datatype %in% 2:3) 2 else 4
+  sign <- if (datatype %in% 0:2) TRUE else FALSE
+  read.data <- function(n.counts){
+    readBin(con, what = what, n = n.counts, size = size, signed = sign,
+            endian = "little")
   }
-
 
   ##loop over all frames
   ##output
@@ -346,7 +333,7 @@ read_SPE2R <- function(
     cat("\n[read_SPE2R()]\n\t >>", file)
 
   ##set progressbar
-  if(txtProgressBar & verbose){
+  if (txtProgressBar) {
     pb<-txtProgressBar(min=0,max=diff(frame.range)+1, char="=", style=3)
   }
 
@@ -365,18 +352,16 @@ read_SPE2R <- function(
     }else{
 
       data.list <- c(data.list, list(temp.data))
-
     }
 
     ##update progress bar
-    if(txtProgressBar & verbose){
+    if (txtProgressBar) {
       setTxtProgressBar(pb, i)
     }
-
   }
 
   ##close
-  if(txtProgressBar & verbose){
+  if (txtProgressBar) {
     close(pb)
     cat("\t >>", i,"records have been read successfully!\n\n")
   }
@@ -389,7 +374,6 @@ read_SPE2R <- function(
 
     data.spectrum.vector <- sapply(1:length(data.list), function(x){
       rowSums(data.list[[x]])
-
     })
 
     ##split vector to matrix
@@ -423,7 +407,6 @@ read_SPE2R <- function(
     object@recordType = "Image"
     object@curveType <- "measured"
     object@info <- temp.info
-
   }
 
   ##close con
@@ -431,5 +414,4 @@ read_SPE2R <- function(
 
   ##return values
   return(object)
-
 }
