@@ -1,15 +1,14 @@
-context("calc_MaxDose")
-
 data(ExampleData.DeValues, envir = environment())
 temp <- calc_MaxDose(ExampleData.DeValues$CA1,
                      sigmab = 0.2,
                      par = 3,
-                     plot = FALSE,
+                     plot = TRUE,
                      verbose = FALSE)
 
 test_that("check class and length of output", {
   testthat::skip_on_cran()
-  expect_equal(is(temp), c("RLum.Results", "RLum"))
+
+  expect_s4_class(temp, "RLum.Results")
   expect_equal(length(temp), 9)
 
 })
@@ -25,7 +24,7 @@ test_that("check values from output example", {
   expect_equal(round(results$ci_lower, digits = 2), 69.65)
   expect_equal(round(results$ci_upper, digits = 2), 99.33)
   expect_equal(results$par, 3)
-  expect_equal(round(results$sig, digits = 2), 0.54)
+  expect_equal(round(results$sig, digits = 2), 1.71)
   expect_equal(round(results$p0, digits = 2), 0.65)
   expect_equal(results$mu, NA)
   expect_equal(round(results$Lmax, digits = 2), -19.79)
