@@ -11,21 +11,30 @@ test_that("Check github_commits()", {
   response <- tryCatch(github_commits(), error = function(e) return(e))
 
   if (inherits(response, "error")){
-    expect_output(print(response), regexp = "status code 403")
+    expect_output(print(response), regexp = "returned status code")
   } else {
     expect_s3_class(response, "data.frame")
   }
 
-  rm(response)
+  tryCatch(expect_error(github_commits(branch = "error"),
+                        "Branch 'error' does not exist"),
+           error = function(e) {
+             expect_output(print(e), "returned status code")
+           })
 })
 
 test_that("Check github_branches()", {
   testthat::skip_on_cran()
 
+  expect_error(github_branches(user = NA),
+               "'user' should be of class 'character'")
+  expect_error(github_branches(repo = NA),
+               "'repo' should be of class 'character'")
+
   response <- tryCatch(github_branches(), error = function(e) return(e))
 
   if (inherits(response, "error")) {
-    expect_output(print(response), regexp = "status code 403")
+    expect_output(print(response), regexp = "returned status code")
  }else {
     expect_s3_class(response, "data.frame")
  }
@@ -36,12 +45,17 @@ test_that("Check github_branches()", {
 test_that("Check github_issues()", {
   testthat::skip_on_cran()
 
+  expect_error(github_issues(user = NA),
+               "'user' should be of class 'character'")
+  expect_error(github_issues(repo = NA),
+               "'repo' should be of class 'character'")
+
   SW({
   response <- tryCatch(github_issues(), error = function(e) return(e))
   })
 
   if (inherits(response, "error")){
-    expect_output(print(response), regexp = "status code 403")
+    expect_output(print(response), regexp = "returned status code")
   }else{
     expect_type(response, "list")
   }
