@@ -2,9 +2,11 @@ test_that("input validation", {
   testthat::skip_on_cran()
 
   expect_error(plot_FilterCombinations("error"),
-               "'filters' should be of type 'list'")
+               "'filters' should be of class 'list'")
+  expect_error(plot_FilterCombinations(list()),
+               "'filters' cannot be an empty list")
   expect_error(plot_FilterCombinations(list("error")),
-               "All elements of 'filter' must be of type")
+               "All elements of 'filters' should be of class 'data.frame'")
   expect_error(plot_FilterCombinations(list(data.frame(a = c(100, 200),
                                                        b = c(0.2, 1.1)))),
                "Transmission values > 1 found, check your data")
